// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI6C500K/24DSI6C500K_1.x.x.x_GSC_DN/samples/savedata/perform.c $
// $Rev: 52725 $
// $Date: 2023-04-03 14:03:10 -0500 (Mon, 03 Apr 2023) $

// 24DSI6C500K: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M				(1024L * 1024L)
#define	TAG_DECODE(v)	(((v) >> 24) & 0x7)



// variables ******************************************************************

static	u32	_buffer[_1M];



//*****************************************************************************
static int _channels(const args_t* args, s32* chans)
{
	int	errs;

	gsc_label("Input Channels");
	errs	= dsi6c500k_query(args->fd, -1, 0, DSI6C500K_QUERY_CHANNEL_QTY, chans);

	if (errs == 0)
		printf("%ld Channels\n", (long) chans[0]);

	return(errs);
}



//*****************************************************************************
static int _read_data(const args_t* args)
{
	char	buf[80];
	int		errs	= 0;
	long	get		= sizeof(_buffer);
	size_t	got;
	int		index;
	int		ret;

	for (index = 0; (errs == 0) && (index < args->repeat);)
	{
		index++;
		strcpy(buf, "Reading");

		if (args->repeat > 1)
		{
			strcat(buf, " #");
			gsc_label_long_comma_buf(index, buf + strlen(buf));
		}

		gsc_label(buf);

		errs	+= dsi6c500k_ai_buf_clear	(args->fd, -1, 0);
		errs	+= dsi6c500k_ai_buf_enable	(args->fd, -1, 0, DSI6C500K_AI_BUF_ENABLE_NO, NULL);
		errs	+= dsi6c500k_ai_buf_clear	(args->fd, -1, 0);
		errs	+= dsi6c500k_ai_buf_overflow(args->fd, -1, 0, DSI6C500K_AI_BUF_OVERFLOW_CLEAR, NULL);
		errs	+= dsi6c500k_ai_buf_enable	(args->fd, -1, 0, DSI6C500K_AI_BUF_ENABLE_YES, NULL);

		if (errs)
		{
			printf("FAIL <---  (setup errors)\n");
			break;
		}

		ret	= dsi6c500k_read_dsl(args->fd, _buffer, get, &got);

		if (ret < 0)
		{
			errs++;
			printf("FAIL <---  (read error: %d)\n", (int) ret);
			break;
		}

		if (got != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (got %ld bytes, requested %ld)\n",
					(long) got,
					(long) get);
			break;
		}

		errs	= 0;
		printf("PASS  (");
		gsc_label_long_comma(get / 4);
		printf(" samples)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _save_eol(const char* name, FILE* file)
{
	int	errs	= 0;
	int	i;

	i	= fprintf(file, "\r\n");

	if (i != 2)
	{
		printf("FAIL <---  (fprintf() failure to %s)\n", name);
		errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
static int _save_value(const args_t* args, const char* name, FILE* file, u32 value)
{
	char	buf1[40];
	char	buf2[80];
	int		errs	= 0;
	int		i;
	int		len;

	if (args->tag_strip)
		value &= 0xFFFFFF;

	if (args->decimal)
		sprintf(buf1, "%ld", (long) value);
	else
		sprintf(buf1, "%08lX", (long) value);

	sprintf(buf2, "  %8s", buf1);
	len	= strlen(buf2);
	i	= fprintf(file, "%8s", buf2);

	if (i != len)
	{
		printf("FAIL <---  (fprintf() failure to %s)\n", name);
		errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
static int _validate_data(int chans, u32 last, long index, u32 data)
{
	int	chan_tag;
	int	errs		= 0;
	int	expect;
	int	last_tag;

	if (data & 0xF8000000)
	{
		errs	= 1;
		printf(" FAIL <---  (Upper bits are invalid: [%ld] = 0x%08lX)\n", index, (long) data);
	}

	chan_tag	= TAG_DECODE(data);

	if (chan_tag >= chans)
	{
		errs	= 1;
		printf(" FAIL <---  (Invalid Channel Tag: [%ld] = 0x%08lX)\n", index, (long) data);
	}

	if (last == 0xFFFFFFFF)
	{
		if (chan_tag != 0)
		{
			errs	= 1;
			printf(" FAIL <---  (First tag not zero: [%ld] = 0x%08lX)\n", index, (long) data);
		}
	}
	else
	{
		last_tag	= TAG_DECODE(last);
		expect		= (last_tag + 1) % chans;

		if (chan_tag != expect)
		{
			errs	= 1;
			printf(	" FAIL <---  (Lost Synchronozation: [%ld] = 0x%08lX, [%ld] = 0x%08lX)\n",
					index - 1,
					(long) last,
					index,
					(long) data);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _save_data(const args_t* args, int chans, int errs)
{
	char		buf[64];
	FILE*		file;
	long		l;
	u32			last		= 0xFFFFFFFF;
	const char*	name		= "data.txt";
	long		samples		= sizeof(_buffer) / 4;
	int			tag_got;

	strcpy(buf, "Saving");

	if (args->force_save)
	{
		errs	= 0;
		strcat(buf, " (Forced)");
	}

	gsc_label(buf);

	for (;;)
	{
		if (errs)
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		for (l = 0; l < samples; l++)
		{
			tag_got	= TAG_DECODE(_buffer[l]);

			if ((l > 0) && (tag_got == 0))
				errs	+= _save_eol(name, file);

			errs	+= _save_value(args, name, file, _buffer[l]);
			errs	+= _validate_data(chans, last, l, _buffer[l]);
			last	= _buffer[l];
		}

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32	chans	= 32;
	int	errs	= 0;

	gsc_label("Capture & Save");
	printf("\n");
	gsc_label_level_inc();

	errs	+= dsi6c500k_config_ai	(args->fd, -1, 1, 1, args->fsamp);
	errs	+= dsi6c500k_ai_mode	(args->fd, -1, 1, args->ai_mode, NULL);
	errs	+= dsi6c500k_rx_io_mode	(args->fd, -1, 1, args->io_mode, NULL);

	if (args->bursting)
	{
		s32		b_size	= 400;		// Burst Size
		s32		b_size_max;			// Burst Block Size - maximum limit
		float	bps;				// Bursts Per Second
		float	bps_min;			// Bursts Per Second - minimum limit
		s32		br_div	= 1000000;	// Burst Rate Divisor
		s32		fclk;				// Burst Rate clock frequency
		s32		rx_to	= 30;

		gsc_label("Bursting");

		errs		+= dsi6c500k_query(args->fd, -1, 0, DSI6C500K_QUERY_CHANNEL_QTY, &chans);
		errs		+= dsi6c500k_query(args->fd, -1, 0, DSI6C500K_QUERY_FCLK_DEFAULT, &fclk);
		errs		+= dsi6c500k_rx_io_timeout(args->fd, -1, 0, -1, &rx_to);
		bps			= (float) fclk / br_div;
		b_size_max	= args->fsamp / bps;

		if (b_size > b_size_max)
			b_size	= b_size_max;

		gsc_label_level_inc();

		if (args->ext_trig)
		{
			bps_min	= (float) 1024 * 1024 / rx_to / chans / b_size;
			printf(	"(External Triggering: ~%.1f minimum Bursts Per Second)\n",
					(float) bps_min);
			errs	+= dsi6c500k_burst_timer(args->fd, -1, 1, DSI6C500K_BURST_TIMER_DISABLE, NULL);
			errs	+= dsi6c500k_sync_src	(args->fd, -1, 1, DSI6C500K_SYNC_SRC_EXT, NULL);
		}
		else
		{
			printf("(Internal Triggering)\n");
			errs	+= dsi6c500k_burst_timer(args->fd, -1, 1, DSI6C500K_BURST_TIMER_ENABLE, NULL);
			errs	+= dsi6c500k_sync_src	(args->fd, -1, 1, DSI6C500K_SYNC_SRC_G0_CA, NULL);
		}

		errs	+= dsi6c500k_burst_rate_div	(args->fd, -1, 1, br_div, NULL);
		errs	+= dsi6c500k_burst_size		(args->fd, -1, 1, b_size, NULL);
		errs	+= dsi6c500k_burst			(args->fd, -1, 1, DSI6C500K_BURST_ENABLE, NULL);

		gsc_label_level_dec();
	}

	errs	+= _channels					(args, &chans);
	errs	+= dsi6c500k_rx_io_overflow		(args->fd, -1, 1, DSI6C500K_IO_OVERFLOW_IGNORE, NULL);
	errs	+= dsi6c500k_rx_io_underflow	(args->fd, -1, 1, DSI6C500K_IO_UNDERFLOW_IGNORE, NULL);
	errs	+= dsi6c500k_ai_buf_clear		(args->fd, -1, 1);
	errs	+= dsi6c500k_ai_buf_overflow	(args->fd, -1, 1, DSI6C500K_AI_BUF_OVERFLOW_CLEAR, NULL);
	errs	+= dsi6c500k_ai_buf_underflow	(args->fd, -1, 1, DSI6C500K_AI_BUF_UNDERFLOW_CLEAR, NULL);
	errs	+= dsi6c500k_rx_io_timeout		(args->fd, -1, 1, 300, NULL);
	errs	+= _read_data(args);
	errs	+= dsi6c500k_ai_buf_overflow	(args->fd, -1, 1, DSI6C500K_AI_BUF_OVERFLOW_TEST, NULL);
	errs	+= dsi6c500k_ai_buf_underflow	(args->fd, -1, 1, DSI6C500K_AI_BUF_UNDERFLOW_TEST, NULL);
	errs	+= _save_data(args, chans, errs);

	gsc_label_level_dec();
	return(errs);
}



